/* This file has been put into the public domain by its author.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
   OTHER DEALINGS IN THE SOFTWARE.
*/
/* fpsplit - split a floating-point number into a sign, exponent, 
   and mantissa
   
   AUTHOR: Gregory Pietsch
   
*/

#include "fp.h"
#include "mpi.h"

void
_FPsplit (_FP_SPLIT_T * r, _FP_T f, _FP_METRIC_T * m)
{
  unsigned long t;
  unsigned long bias;
  size_t i;

  /* get sign */
  r->_S = _Getsign (f, m);
  /* get exponent */
  for (i = 0, t = 0; i < 3; ++i)
    t = (t << CHAR_BIT) | f[_BYTE (i, m->_Size)];
  t &= ~(~0UL << (3 * CHAR_BIT - 1));
  t >>= (3 * CHAR_BIT - 1 - m->_Size);
  r->_E = t;
  /* get mantissa */
  _Lsu (r->_M, f, m->_Ebits + 1, m->_Size);
  if (m->_Explicit1bit == 0)
    {
      _Rsu (r->_M, r->_M, 1, m->_Size);
      r->_M[_BYTE (0, m->_Size)] |= (1 << (CHAR_BIT - 1));
    }
}

/* END OF FILE */
