/* This file has been put into the public domain by its author.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
   OTHER DEALINGS IN THE SOFTWARE.
*/
/* mlc.c - Multiply two complex numbers

   AUTHOR: Gregory Pietsch
   
*/

#include "fp.h"
#include "mpi.h"

_FP_T
_Mlc (_FP_T c, _FP_T a, _FP_T b, const _FP_METRIC_T *const m)
{
  unsigned char x[m->_Size << 1];
  unsigned char t0[m->_Size], t1[m->_Size], t2[m->_Size];

  if (_Classify (a + m->_Size, m) == FP_ZERO)
    {
      _Mlf (t0, a, b, m);
      _Mlf (t1, a, b + m->_Size, m);
      _Cpy (x, t0, m->_Size);
      _Cpy (x + m->_Size, t1, m->_Size);
    }
  else if (_Classify (b + m->_Size, m) == FP_ZERO)
    {
      _Mlf (t0, a, b, m);
      _Mlf (t1, a + m->_Size, b, m);
      _Cpy (x, t0, m->_Size);
      _Cpy (x + m->_Size, t1, m->_Size);
    }
  else
    {
      _Adf (t0, a, a + m->_Size, m);
      _Adf (t1, b, b + m->_Size, m);
      _Mlf (t2, t0, t1, m);
      _Mlf (t0, a, b, m);
      _Mlf (t1, a + m->_Size, b + m->_Size, m);
      _Cpy (x, t0, m->_Size);
      _Sbf (x, t1, m);
      _Cpy (x + m->_Size, t2, m->_Size);
      _Sbf (x + m->_Size, t0, m);
      _Sbf (x + m->_Size, t1, m);
    }
  _Cpy (c, x, m->_Size << 1);
  return c;
}

/* END OF FILE */
